@@ -157,6 +157,9 @@ def _qtwebengine_features(
         # WORKAROUND for https://bugreports.qt.io/browse/QTBUG-89740
         disabled_features.append('InstalledApp')
 
+    if not config.val.input.media_keys:
+        disabled_features.append('HardwareMediaKeyHandling')
+
     return (enabled_features, disabled_features)
 
 
@@ -331,7 +334,11 @@ def _qtwebengine_settings_args(versions: version.WebEngineVersions) -> Iterator[
         },
         'content.headers.referer': {
             'always': None,
-        }
+        },
+        'content.prefers_reduced_motion': {
+            True: '--force-prefers-reduced-motion',
+            False: None,
+        },
     }
     qt_514_ver = utils.VersionNumber(5, 14)
 
