@@ -32,8 +32,6 @@ import functools
 import contextlib
 import shlex
 import mimetypes
-import ctypes
-import ctypes.util
 from typing import (Any, Callable, IO, Iterator,
                     Optional, Sequence, Tuple, Type, Union,
                     TypeVar, TYPE_CHECKING)
@@ -89,7 +87,7 @@ class VersionNumber:
     """A representation of a version number."""
 
     def __init__(self, *args: int) -> None:
-        self._ver = QVersionNumber(*args)
+        self._ver = QVersionNumber(args)  # not *args, to support >3 components
         if self._ver.isNull():
             raise ValueError("Can't construct a null version")
 
@@ -104,8 +102,6 @@ class VersionNumber:
         self.patch = self._ver.microVersion()
         self.segments = self._ver.segments()
 
-        assert len(self.segments) <= 3, self.segments
-
     def __str__(self) -> str:
         return ".".join(str(s) for s in self.segments)
 
@@ -345,7 +341,7 @@ class prevent_exceptions:  # noqa: N801,N806 pylint: disable=invalid-name
         self._retval = retval
         self._predicate = predicate
 
-    def __call__(self, func: Callable) -> Callable:
+    def __call__(self, func: Callable[..., Any]) -> Callable[..., Any]:
         """Called when a function should be decorated.
 
         Args:
@@ -433,7 +429,7 @@ def qualname(obj: Any) -> str:
 _ExceptionType = Union[Type[BaseException], Tuple[Type[BaseException]]]
 
 
-def raises(exc: _ExceptionType, func: Callable, *args: Any) -> bool:
+def raises(exc: _ExceptionType, func: Callable[..., Any], *args: Any) -> bool:
     """Check if a function raises a given exception.
 
     Args:
@@ -711,7 +707,10 @@ def yaml_dump(data: Any, f: IO[str] = None) -> Optional[str]:
         return yaml_data.decode('utf-8')
 
 
-def chunk(elems: Sequence, n: int) -> Iterator[Sequence]:
+_T = TypeVar('_T')
+
+
+def chunk(elems: Sequence[_T], n: int) -> Iterator[Sequence[_T]]:
     """Yield successive n-sized chunks from elems.
 
     If elems % n != 0, the last chunk will be smaller.
@@ -753,19 +752,6 @@ def ceil_log(number: int, base: int) -> int:
     return result
 
 
-def libgl_workaround() -> None:
-    """Work around QOpenGLShaderProgram issues, especially for Nvidia.
-
-    See https://bugs.launchpad.net/ubuntu/+source/python-qt4/+bug/941826
-    """
-    if os.environ.get('QUTE_SKIP_LIBGL_WORKAROUND'):
-        return
-
-    libgl = ctypes.util.find_library("GL")
-    if libgl is not None:  # pragma: no branch
-        ctypes.CDLL(libgl, mode=ctypes.RTLD_GLOBAL)
-
-
 def parse_duration(duration: str) -> int:
     """Parse duration in format XhYmZs into milliseconds duration."""
     if duration.isdigit():
